/**
 * 
 */
package marf.net.server.corba.Classification;

import marf.Classification.ClassificationException;
import marf.net.CommunicationException;
import marf.net.server.corba.CORBACommunicationException;
import marf.net.server.corba.ICORBAServer;
import marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBA;
import marf.net.server.corba.Storage.Configuration;
import marf.net.server.corba.Storage.Result;
import marf.net.server.corba.Storage.ResultSet;
import marf.net.server.corba.util.MARFObjectAdapter;
import marf.net.server.corba.util.ORBUtils;
import marf.net.server.delegates.Classification.BasicClassificationDelegate;
import marf.net.server.delegates.Classification.IClassificationDelegate;
import marf.util.Arrays;
import marf.util.Logger;
import marf.util.MARFException;

import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.PortableServer.POA;


/**
 * CORBA-specific Classification Servant.
 * 
 * All transactional semantics, inter-server communication,
 * and business logic is delegated to the enclosed delegate
 * instance.
 * 
 * @author mokhov
 *
 */
public class ClassificationServant
extends IClassificationCORBAPOA
implements ICORBAServer
{
	/**
	 * Some classification transaction and business logic delegate.
	 */
	private IClassificationDelegate oDelegate = null;

	/**
	 * Cannot delegate this as it is CORBA-specific. 
	 */
	private IFeatureExtractionCORBA oFeatureExtractionRemote = null;
	
	/**
	 * The root POA of this servant. 
	 */
	private POA oRootPOA = null;

	
	/**
	 * @throws ClassificationException
	 */
	public ClassificationServant()
	throws ClassificationException, Exception
	{
		super();
		new Logger("classification.corba.log");
		
	//	try
		{
			this.oDelegate = new BasicClassificationDelegate();
		}
		/*catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new RuntimeException(e);
		}*/
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#init(int)
	 */
	public boolean init(Configuration poConfig)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.init(MARFObjectAdapter.getMARFConfiguration(poConfig));
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#classify()
	 */
	public boolean classify()
	throws CORBACommunicationException
	{
		try
		{
//			return this.oDelegate.classify();
			double[] adFeatureVector = this.oFeatureExtractionRemote.getFeaturesArray();
			System.out.println("Got " + adFeatureVector.length + " features from the remote FE module.");
			return this.oDelegate.classify(adFeatureVector);
		}
		catch(ClassificationException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#train()
	 */
	public boolean train()
	throws CORBACommunicationException
	{
		try
		{
			double[] adFeatureVector = this.oFeatureExtractionRemote.getFeaturesArray();
			System.out.println("Got " + adFeatureVector.length + " features from the remote FE module.");
			return this.oDelegate.train(adFeatureVector);
		}
		catch(ClassificationException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#classifyFromFeatureVector(double[])
	 */
	public boolean classifyFromFeatureVector(double[] padFeatureVector)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.classify(padFeatureVector);
		}
		catch(ClassificationException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#trainFromFeatureVector(double[])
	 */
	public boolean trainFromFeatureVector(double[] padFeatureVector)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.train(padFeatureVector);
		}
		catch(ClassificationException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#getResult()
	 */
	public Result getResult()
	{
		return MARFObjectAdapter.getCORBAResult(this.oDelegate.getResult());
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#getResultSet()
	 */
	public ResultSet getResultSet()
	{
		return MARFObjectAdapter.getCORBAResultSet(this.oDelegate.getResultSet());
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#getFeatureExtraction()
	 */
	public IFeatureExtractionCORBA getFeatureExtraction()
	{
		return this.oFeatureExtractionRemote;
	}

	/**
	 * @see marf.net.server.corba.Classification.IClassificationCORBAOperations#setFeatureExtraction(marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBA)
	 */
	public void setFeatureExtraction(IFeatureExtractionCORBA poFeatureExtraction)
	{
		this.oFeatureExtractionRemote = poFeatureExtraction;
	}

	
	
	/**
	 * @see marf.net.server.corba.ICORBAServer#getRootPOA()
	 */
	public POA getRootPOA()
	{
		return this.oRootPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#setRootPOA(org.omg.PortableServer.POA)
	 */
	public void setRootPOA(POA poPOA)
	{
		this.oRootPOA = poPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#register(org.omg.CosNaming.NamingContext)
	 */
	public void register(NamingContext poNamingContext)
	throws Exception
	{
  		org.omg.CORBA.Object oCORBAServantObject = this.oRootPOA.servant_to_reference(this);
  		IClassificationCORBA oServantInterface = IClassificationCORBAHelper.narrow(oCORBAServantObject);

        // Bind the Object Reference in Naming
        NameComponent oNameComponent = new NameComponent("IClassificationCORBA", "");
        NameComponent[] aoPath = {oNameComponent};
        poNamingContext.rebind(aoPath, oServantInterface);
	}

	
	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		String[] astrORBArgv =
		{
			"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_CLASSIFICATION_PORT),
			"-ORBInitialHost", ORBUtils.getClassificationHost()
		};

		ORBUtils.startORBClassificationService(Arrays.concatenate(astrORBArgv, argv));
	}
}

// EOF
