/**
 * 
 */
package marf.net.server.delegates.Classification;

import java.io.Serializable;

import marf.Configuration;
import marf.MARF;
import marf.Classification.ClassificationException;
import marf.Classification.ClassificationFactory;
import marf.Classification.IClassification;
import marf.FeatureExtraction.IFeatureExtraction;
import marf.Storage.Result;
import marf.Storage.ResultSet;
import marf.Storage.StorageManager;
import marf.util.Debug;
import marf.util.MARFException;


/**
 * Basic Classification delegate offers inter-server communication
 * and business logic delegation. While all the methods are wrapped
 * into the transactional methods, the latter are not implemented
 * in the basic version. The transactional semantics is implemented
 * in the Recoverable delegate.
 * 
 * @author Serguei Mokhov
 */
public class BasicClassificationDelegate
implements IClassificationDelegate
{
	/**
	 * 
	 */
	protected IClassification oBusinessLogicDelegate = null;

	/**
	 * @throws ClassificationException
	 */
	public BasicClassificationDelegate()
	throws ClassificationException
	{
		super();
	}

	/**
	 * @see marf.net.server.delegates.Classification.IClassificationDelegate#init(marf.Configuration)
	 */
	public boolean init(Configuration poConfig)
	throws MARFException
	{
		MARF.setConfiguration(poConfig);
		this.oBusinessLogicDelegate = ClassificationFactory.create(MARF.getClassificationMethod(), null);
		return true;
	}

	/**
	 * @see marf.Classification.IClassification#classify()
	 */
	public synchronized boolean classify()
	throws ClassificationException
	{
/*
		boolean bRetval = false;

		long iTxnID = -1;
		
		try
		{
			iTxnID = beginTransaction();
			{
				prepareTransaction(iTxnID, (Serializable)this.oClassifcationBusinessLogic);
				bRetval = this.oClassifcationBusinessLogic.classify();
				preliminaryCompleteTransaction(iTxnID, (Serializable)this.oClassifcationBusinessLogic);
			}
			commitTransaction(iTxnID);
		}
		catch(Exception e)
		{
			abortTransaction(iTxnID);

			e.printStackTrace(System.err);
			bRetval = false;
		}
		
		return bRetval;
*/
		boolean bRetval = false;

//		long iTxnID = requestTransaction("classify", (Serializable)this.oClassifcationBusinessLogic);
		long iTxnID = requestTransaction("classify", this.oBusinessLogicDelegate);
		{
			bRetval = this.oBusinessLogicDelegate.classify();
		}
//		endTransaction(iTxnID, (Serializable)this.oClassifcationBusinessLogic);
		endTransaction(iTxnID, this.oBusinessLogicDelegate);
		
		return bRetval;
	}

	public Serializable getSerializableValue(Object poValue)
	{
		Serializable oValue = null;
		
		// Special case for MARF's StorageManager instances
		if(poValue instanceof StorageManager)
		{
			StorageManager oManager = (StorageManager)poValue;
			
			// Attempt to store the instance of the managed Serializable object if available
			if(oManager.getObjectToSerialize() != null)
			{
				oValue = oManager.getObjectToSerialize();
			}
			
			// If not, store the state of the manager itself (which is also Serializable)
			else
			{
				oValue = oManager;
			}
		}

		// Generally, anything serializable
		else if(poValue instanceof Serializable)
		{
			oValue = (Serializable)poValue;
		}

		// Else report an error
		else
		{
			throw new RuntimeException("Class " + poValue.getClass().getName() + " is not marked as Serializable.");
		}
		
		return oValue;
	}
	
//	public synchronized long requestTransaction(String pstrOperation, Serializable poValue)
	public synchronized long requestTransaction(String pstrOperation, Object poValue)
	{
		long iTxnID = -1;
		
		Serializable oValue = getSerializableValue(poValue);
		
		try
		{
			iTxnID = beginTransaction(pstrOperation);
			{
				prepareTransaction(iTxnID, oValue);
			}
		}
		catch(Throwable e)
		{
			e.printStackTrace(System.err);

			endTransaction(iTxnID, oValue);
			
			throw new RuntimeException(e);
		}
		
		return iTxnID;
	}
	
//	public synchronized boolean endTransaction(long iTxnID, Serializable poValue)
	public synchronized boolean endTransaction(long iTxnID, Object poValue)
	{
		boolean bRetval = false;

		Serializable oValue = getSerializableValue(poValue);

		try
		{
			{
				preliminaryCompleteTransaction(iTxnID, oValue);
			}
			commitTransaction(iTxnID);
			
			bRetval = true;
		}
		catch(Exception e)
		{
			abortTransaction(iTxnID);
		
			e.printStackTrace(System.err);
			bRetval = false;
		}
		
		return bRetval;
	}
	
	public void preliminaryCompleteTransaction(long plTxnID, Serializable poValue)
	{
		prepareTransaction(plTxnID, poValue);
		// TODO
	}
	
	/**
	 * @see marf.Classification.IClassification#classify(double[])
	 */
	public synchronized boolean classify(double[] padFeatureVector)
	throws ClassificationException
	{
		return this.oBusinessLogicDelegate.classify(padFeatureVector);
	}

	/**
	 * @see marf.Classification.IClassification#train(double[])
	 */
	public synchronized boolean train(double[] padFeatureVector)
	throws ClassificationException
	{
		return this.oBusinessLogicDelegate.train(padFeatureVector);
	}

	/**
	 * @see marf.Classification.IClassification#getFeatureExtraction()
	 */
	public synchronized IFeatureExtraction getFeatureExtraction()
	{
		return this.oBusinessLogicDelegate.getFeatureExtraction();
	}

	/**
	 * @see marf.Classification.IClassification#getResult()
	 */
	public synchronized Result getResult()
	{
		return this.oBusinessLogicDelegate.getResult();
	}

	/**
	 * @see marf.Classification.IClassification#getResultSet()
	 */
	public synchronized ResultSet getResultSet()
	{
		return this.oBusinessLogicDelegate.getResultSet();
	}

	/**
	 * @see marf.Classification.IClassification#setFeatureExtraction(marf.FeatureExtraction.IFeatureExtraction)
	 */
	public synchronized void setFeatureExtraction(IFeatureExtraction poFeatureExtraction)
	{
		this.oBusinessLogicDelegate.setFeatureExtraction(poFeatureExtraction);
	}

	/**
	 * @see marf.Classification.IClassification#train()
	 */
	public boolean train()
	throws ClassificationException
	{
		boolean bRetval = false;

		long iTxnID = -1;
		
		try
		{
			iTxnID = beginTransaction("train");
			{
				prepareTransaction(iTxnID, (Serializable)this.oBusinessLogicDelegate);
				bRetval = this.oBusinessLogicDelegate.train();
			}
			commitTransaction(iTxnID);
		}
		catch(Exception e)
		{
			abortTransaction(iTxnID);

			e.printStackTrace(System.err);
			bRetval = false;
		}
		
		return bRetval;
	}

	/**
	 * Does nothing in basic delegate.
	 * @return true
	 * @see marf.net.server.delegates.IDelegate#abortTransaction(long)
	 */
	public boolean abortTransaction(long piTxnID)
	{
		Debug.debug("Basic transaction " + piTxnID + " aborted (fake).");
		return true;
	}

	/**
	 * Does nothing in basic delegate.
	 * @return 0
	 * @see marf.net.server.delegates.IDelegate#beginTransaction()
	 */
	public long beginTransaction(String pstrOperation)
	{
		Debug.debug("Basic transaction 0 initiated (fake) for " + pstrOperation);
		return 0;
	}

	/**
	 * Does nothing in basic delegate.
	 * @return true
	 * @see marf.net.server.delegates.IDelegate#commitTransaction(long)
	 */
	public boolean commitTransaction(long piTxnID)
	{
		Debug.debug("Basic transaction " + piTxnID + " comitted (fake).");
		return true;
	}

	/**
	 * Does nothing in basic delegate.
	 * @return true
	 * @see marf.net.server.delegates.IDelegate#prepareTransaction(long, java.io.Serializable)
	 */
	public boolean prepareTransaction(long piTxnID, Serializable poValue)
	{
		Debug.debug(new StringBuffer("Basic transaction ").append(piTxnID).append(" for ").append(poValue).append(" prepared (fake)."));
		return true;
	}
}

// EOF
