package marf.net.server.delegates.MARF;

import marf.Configuration;
import marf.Classification.IClassification;
import marf.FeatureExtraction.IFeatureExtraction;
import marf.Preprocessing.IPreprocessing;
import marf.Storage.ISampleLoader;
import marf.Storage.ModuleParams;
import marf.Storage.Result;
import marf.Storage.ResultSet;
import marf.Storage.Sample;
import marf.net.server.delegates.IDelegate;
import marf.nlp.NLPException;
import marf.util.MARFException;
import marf.util.NotImplementedException;


/**
 * @author mokhov
 *
 */
public interface IMARFServerDelegate
extends IDelegate
{
	void setPreprocessingMethod(final int piPreprocessingMethod)
	throws MARFException;

	/**
	 * Gets currently selected preprocessing method.
	 * @return one of the preprocessing methods
	 */
	int getPreprocessingMethod();

	/**
	 * Sets feature extraction method to be used.
	 * @param piFeatureExtractionMethod one of the allowed feature extraction methods
	 * @throws MARFException if the parameter outside of the valid range
	 */
	void setFeatureExtractionMethod(final int piFeatureExtractionMethod)
	throws MARFException;

	/**
	 * Gets currently selected feature extraction method.
	 * @return current feature extraction method
	 */
	int getFeatureExtractionMethod();

	/**
	 * Sets classification method to be used.
	 * @param piClassificationMethod one of the allowed classification methods
	 * @throws MARFException if the parameter outside of the valid range
	 */
	void setClassificationMethod(final int piClassificationMethod)
	throws MARFException;

	/**
	 * Gets classification method to be used.
	 * @return current classification method
	 */
	int getClassificationMethod();

	/**
	 * Sets input sample file format.
	 * @param piSampleFormat one of the allowed sample formats
	 */
	void setSampleFormat(final int piSampleFormat);

	/**
	 * Gets input sample file format.
	 * @return current sample format
	 */
	int getSampleFormat();

	/**
	 * Sets input sample file name.
	 * @param pstrFileName string representing sample file to be read
	 */
	void setSampleFile(final String pstrFileName);

	/**
	 * Obtains filename of a sample currently being processed.
	 * @return file name of a string representing sample file
	 */
	String getSampleFile();

	/**
	 * Sets directory with sample files to be read from.
	 * @param pstrSamplesDir string representing directory name
	 */
	void setSamplesDir(final String pstrSamplesDir);

	/**
	 * Sets module-specific parameters an application programmer wishes to pass on to the module.
	 * @param poModuleParams parameters' instance
	 */
	void setModuleParams(final ModuleParams poModuleParams);

	/**
	 * Gets module-specific parameters an application programmer passed on to the module.
	 * @return ModuleParams object reference
	 */
	ModuleParams getModuleParams();

	/**
	 * Indicates whether spectrogram is wanted as an output of a FeatureExtraction module.
	 * @param pbDump <code>true</code> if wanted, <code>false</code> if not
	 */
	void setDumpSpectrogram(final boolean pbDump);

	/**
	 * Whether spectrogram wanted or not.
	 * @return <code>true</code> if spectrogram being dumped, <code>false</code> otherwise
	 */
	boolean getDumpSpectrogram();

	/**
	 * Indicates whether wave graph is wanted as an output.
	 * @param pbDump <code>true</code> if wanted, <code>false</code> if not
	 */
	void setDumpWaveGraph(final boolean pbDump);

	/**
	 * Whether wave graph wanted or not.
	 * @return <code>true</code> if graph wanted being dumped, <code>false</code> otherwise
	 */
	boolean getDumpWaveGraph();

	/**
	 * Sets ID of a subject currently being trained on.
	 * @param piSubjectID integer ID of the subject
	 */
	void setCurrentSubject(final int piSubjectID);

	/**
	 * Gets ID of a subject currently being trained on.
	 * @return integer ID of the subject
	 * @since 0.2.0
	 */
	int getCurrentSubject();

	/**
	 * Allows loading a sample loader plugin by its name.
	 * @param pstrClassName class name of the plugin sample loader module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @since 0.3.0.5
	 */
	void setSampleLoaderPluginClass(String pstrClassName)
	throws MARFException;

	/**
	 * Allows setting a loaded sample loader plugin class.
	 * @param poClass class represting a sample loader plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @since 0.3.0.5
	 */
	void setSampleLoaderPluginClass(Class poClass)
	throws MARFException;

	/**
	 * Allows querying for the current preprocessing plugin class.
	 * @return the internal plugin class
	 * @since 0.3.0.5
	 */
	Class getSampleLoaderPluginClass();

	/**
	 * Allows loading a preprocessing plugin by its name.
	 * @param pstrClassName class name of the plugin preprocessing module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @since 0.3.0.4
	 */
	void setPreprocessingPluginClass(String pstrClassName)
	throws MARFException;

	/**
	 * Allows setting a loaded preprocessing plugin class.
	 * @param poClass class represting a preprocessing plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @since 0.3.0.4
	 */
	void setPreprocessingPluginClass(Class poClass)
	throws MARFException;

	/**
	 * Allows querying for the current preprocessing plugin class.
	 * @return the internal plugin class
	 * @since 0.3.0.4
	 */
	Class getPreprocessingPluginClass();

	/**
	 * Allows loading a feature extraction plugin by its name.
	 * @param pstrClassName class name of the plugin feature extraction module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @since 0.3.0.4
	 */
	void setFeatureExtractionPluginClass(String pstrClassName)
	throws MARFException;

	/**
	 * Allows setting a loaded feature extraction class plugin class.
	 * @param poClass class represting a feature extraction plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @since 0.3.0.4
	 */
	void setFeatureExtractionPluginClass(Class poClass)
	throws MARFException;

	/**
	 * Allows querying for the current feature extraction plugin class.
	 * @return the internal plugin class
	 * @since 0.3.0.4
	 */
	Class getFeatureExtractionPluginClass();

	/**
	 * Allows loading a classification plugin by its name.
	 * @param pstrClassName class name of the plugin classification module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @since 0.3.0.4
	 */
	void setClassificationPluginClass(String pstrClassName)
	throws MARFException;

	/**
	 * Allows setting a loaded classification plugin class.
	 * @param poClass class represting a classification plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @since 0.3.0.4
	 */
	void setClassificationPluginClass(Class poClass)
	throws MARFException;

	/**
	 * Allows querying for the current classification plugin class.
	 * @return the internal plugin class
	 * @since 0.3.0.4
	 */
	Class getClassificationPluginClass();

	/**
	 * Returns a string representation of the MARF version.
	 * As of 0.3.0.3 MINOR_REVISION is also returned.
	 * 
	 * @return version String
	 * @see #MINOR_REVISION
	 */
	String getVersion();

	/**
	 * Returns an integer representation of the MARF version.
	 * As of 0.3.0.3, MINOR_REVISION is included into calculations
	 * and the formula changed to begin with 1000 as a MAJOR_VERSION
	 * coefficient.
	 *
	 * @return integer version as <code>MAJOR_VERSION * 1000 + MINOR_VERSION * 100 + REVISION * 10 + MINOR_REVISION</code>
	 *
	 * @see #MAJOR_VERSION
	 * @see #MINOR_VERSION
	 * @see #REVISION
	 * @see #MINOR_REVISION
	 */
	int getIntVersion();

	/**
	 * Retrieves double version of MARF. Unlike the integer version, the double
	 * one begins with 100 and the minor revision is returned after the point,
	 * e.g. 123.4 for 1.2.3.4.
	 *
	 * @return double version as <code>MAJOR_VERSION * 100 + MINOR_VERSION * 10 + REVISION + MINOR_REVISION / 10</code>
	 *
	 * @see #MAJOR_VERSION
	 * @see #MINOR_VERSION
	 * @see #REVISION
	 * @see #MINOR_REVISION
	 *
	 * @since 0.3.0.3
	 */
	double getDoubleVersion();

	/**
	 * Returns a string representation of the current MARF configuration.
	 * @return configuration string
	 */
	String getConfig();

	/**
	 * Retrieves current <code>Sample</code> reference.
	 * @return Sample object
	 * @since 0.2.0
	 */
	Sample getSample();

	/**
	 * Retrieves current <code>SampleLoader</code> reference.
	 * @return SampleLoader object
	 * @since 0.2.0
	 */
	ISampleLoader getSampleLoader();

	/**
	 * Retrieves current <code>Preprocessing</code> reference.
	 * @return Preprocessing object
	 * @since 0.2.0
	 */
	IPreprocessing getPreprocessing();

	/**
	 * Retrieves current <code>FeatureExtraction</code> reference.
	 * @return FeatureExtraction object
	 * @since 0.2.0
	 */
	IFeatureExtraction getFeatureExtraction();

	/**
	 * Retrieves current <code>Classification</code> reference.
	 * @return Classification object
	 * @since 0.2.0
	 */
	IClassification getClassification();

	/**
	 * Queries for the final classification result.
	 * @return integer ID of the indentified subject
	 */
	int queryResultID();

	/**
	 * Gets the entire Result object of the likely outcome.
	 * @return Result ID and all the stats of the classification
	 */
	Result getResult();

	/**
	 * Gets the entire collection of results.
	 * @return ResultSet object with one or more results.
	 * @since 0.3.0.2
	 */
	ResultSet getResultSet();

	/* API */

	/**
	 * Recognition/Identification mode.
	 * @since 0.2.0
	 * @throws MARFException if there was an error in the pipeline
	 * or classification
	 */
	void recognize()
	throws MARFException;

	/**
	 * Training mode.
	 * @since 0.2.0
	 * @throws MARFException if the subject is unset or there was
	 * an error in training in the underlying classification module
	 */
	void train()
	throws MARFException;

	/**
	 * The core processing pipeline. The pipeline work
	 * through almost entire process up until creation
	 * of the classification module. Then the application's
	 * choice depicts whether to call train() or classify()
	 * on this module via MARF's train() or recognize() methods.
	 * The pipeline is granularly synchronized on each object
	 * that gets instantiated, so no indirect blocking occurs
	 * on MARF itself (e.g. betwen the main thread and the inner
	 * threads of the framework that maybe accessing the MARF
	 * class concurrently).
	 *
	 * @throws MARFException in case any underlying error happens
	 * @see #train()
	 * @see #recognize()
	 */
	void startRecognitionPipeline()
	throws MARFException;

	/**
	 * Checks for all necessary settings to be present.
	 * Specifically, checks whether preprocessing, feature
	 * extraction, classification methods are set as well as
	 * the audio sample format. The filename of the sample
	 * or a directory must also be present.
	 *
	 * @throws MARFException if any of the settings are unset
	 * @since 0.3.0.5
	 */
	void checkSettings()
	throws MARFException;
	
	/**
	 * Meant to provide implementation of the buffered sample processing for large samples.
	 * Not implemented.
	 * @throws NotImplementedException
	 */
	void streamedRecognition();

	Configuration getConfiguration();
	Configuration setConfiguration(Configuration poConfig)
	throws MARFException;

	/**
	 * <p>Class NLP is more related to the Natural Language Processing
	 * part of MARF.</p>
	 *
	 * @author Serguei Mokhov
	 * @since 0.3.0.2
	 */
	public interface INLPServerDelegate
	{

		/**
		 * Retrieves current smoothing method.
		 * @return inner smoothing method
		 */
		int getSmoothingMethod();

		/**
		 * Sets current smoothing method.
		 * @param piSmoothingMethod new smoothing method to use
		 * @throws NLPException if the parameter is outside the valid range
		 * @see EStatisticalEstimators#MIN_STATS_ESTIMATOR
		 * @see EStatisticalEstimators#MAX_STATS_ESTIMATOR
		 */
		void setSmoothingMethod(final int piSmoothingMethod)
		throws NLPException;

		/**
		 * Retrieves current n-gram modeld.
		 * @return inner n-gram model
		 */
		int getNgramModel();

		/**
		 * Sets current n-gram model.
		 * @param piNgramModel new n-gram model to use
		 * @throws NLPException if the parameter is outside the valid range
		 * @see ENgramModels#MIN_NGRAM_MODEL
		 * @see ENgramModels#MAX_NGRAM_MODEL
		 */
		void setNgramModel(final int piNgramModel)
		throws NLPException;

		/**
		 * Retrieves current language being processed.
		 * @return inner smoothing method
		 */
		String getLanguage();

		/**
		 * Sets current processed language.
		 * @param pstrLanguages new language value
		 * @throws NLPException if the parameter is null or empty
		 */
		void setLanguage(String pstrLanguages)
		throws NLPException;
	} // NLP interface
}

// EOF
