package marf.net.server.rmi.MARF;

import java.rmi.RemoteException;

import marf.Configuration;
import marf.Classification.IClassification;
import marf.FeatureExtraction.IFeatureExtraction;
import marf.MARF.ENgramModels;
import marf.MARF.EStatisticalEstimators;
import marf.Preprocessing.IPreprocessing;
import marf.Storage.ISampleLoader;
import marf.Storage.ModuleParams;
import marf.Storage.Result;
import marf.Storage.ResultSet;
import marf.Storage.Sample;
import marf.net.server.rmi.IRMIServer;
import marf.net.server.rmi.Classification.IClassificationRMI;
import marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI;
import marf.net.server.rmi.Preprocessing.IPreprocessingRMI;
import marf.net.server.rmi.Storage.ISampleLoaderRMI;
import marf.nlp.NLPException;
import marf.util.MARFException;
import marf.util.NotImplementedException;


/**
 * <p>Main RMI MARF Server Interface.
 * </p>
 *
 * @author Serguei Mokhov
 * @since 0.3.0.6
 * @version $Revision: 1.4 $
 */
public interface IMARFServerRMI
extends IRMIServer
{
	/**
	 * @param piPreprocessingMethod
	 * @throws RemoteException
	 * @throws MARFException
	 */
	void setPreprocessingMethod(final int piPreprocessingMethod)
	throws RemoteException, MARFException;

	/**
	 * Gets currently selected preprocessing method.
	 * @return one of the preprocessing methods
	 * @throws RemoteException
	 */
	int getPreprocessingMethod()
	throws RemoteException;

	/**
	 * Sets feature extraction method to be used.
	 * @param piFeatureExtractionMethod one of the allowed feature extraction methods
	 * @throws MARFException if the parameter outside of the valid range
	 * @throws RemoteException
	 */
	void setFeatureExtractionMethod(final int piFeatureExtractionMethod)
	throws RemoteException, MARFException;

	/**
	 * Gets currently selected feature extraction method.
	 * @return current feature extraction method
	 * @throws RemoteException
	 */
	int getFeatureExtractionMethod()
	throws RemoteException;

	/**
	 * Sets classification method to be used.
	 * @param piClassificationMethod one of the allowed classification methods
	 * @throws MARFException if the parameter outside of the valid range
	 * @throws RemoteException
	 */
	void setClassificationMethod(final int piClassificationMethod)
	throws RemoteException, MARFException;

	/**
	 * Gets classification method to be used.
	 * @return current classification method
	 * @throws RemoteException
	 */
	int getClassificationMethod()
	throws RemoteException;

	/**
	 * Sets input sample file format.
	 * @param piSampleFormat one of the allowed sample formats
	 * @throws RemoteException
	 */
	void setSampleFormat(final int piSampleFormat)
	throws RemoteException;

	/**
	 * Gets input sample file format.
	 * @return current sample format
	 * @throws RemoteException
	 */
	int getSampleFormat()
	throws RemoteException;

	/**
	 * Sets input sample file name.
	 * @param pstrFileName string representing sample file to be read
	 * @throws RemoteException
	 */
	void setSampleFile(final String pstrFileName)
	throws RemoteException;

	/**
	 * @param pstrFileName
	 * @param patSampleData
	 * @throws RemoteException
	 */
	void setSampleFile(String pstrFileName, byte[] patSampleData)
	throws RemoteException;
	
	/**
	 * Obtains filename of a sample currently being processed.
	 * @return file name of a string representing sample file
	 * @throws RemoteException
	 */
	String getSampleFile()
	throws RemoteException;

	/**
	 * Sets directory with sample files to be read from.
	 * @param pstrSamplesDir string representing directory name
	 * @throws RemoteException
	 */
	void setSamplesDir(final String pstrSamplesDir)
	throws RemoteException;

	/**
	 * Sets module-specific parameters an application programmer wishes to pass on to the module.
	 * @param poModuleParams parameters' instance
	 * @throws RemoteException
	 */
	void setModuleParams(final ModuleParams poModuleParams)
	throws RemoteException;

	/**
	 * Gets module-specific parameters an application programmer passed on to the module.
	 * @return ModuleParams object reference
	 * @throws RemoteException
	 */
	ModuleParams getModuleParams()
	throws RemoteException;

	/**
	 * Indicates whether spectrogram is wanted as an output of a FeatureExtraction module.
	 * @param pbDump <code>true</code> if wanted, <code>false</code> if not
	 * @throws RemoteException
	 */
	void setDumpSpectrogram(final boolean pbDump)
	throws RemoteException;

	/**
	 * Whether spectrogram wanted or not.
	 * @return <code>true</code> if spectrogram being dumped, <code>false</code> otherwise
	 * @throws RemoteException
	 */
	boolean getDumpSpectrogram()
	throws RemoteException;

	/**
	 * Indicates whether wave graph is wanted as an output.
	 * @param pbDump <code>true</code> if wanted, <code>false</code> if not
	 * @throws RemoteException
	 */
	void setDumpWaveGraph(final boolean pbDump)
	throws RemoteException;

	/**
	 * Whether wave graph wanted or not.
	 * @return <code>true</code> if graph wanted being dumped, <code>false</code> otherwise
	 * @throws RemoteException
	 */
	boolean getDumpWaveGraph()
	throws RemoteException;

	/**
	 * Sets ID of a subject currently being trained on.
	 * @param piSubjectID integer ID of the subject
	 * @throws RemoteException
	 */
	void setCurrentSubject(final int piSubjectID)
	throws RemoteException;

	/**
	 * Gets ID of a subject currently being trained on.
	 * @return integer ID of the subject
	 * @throws RemoteException
	 */
	int getCurrentSubject()
	throws RemoteException;

	/**
	 * Allows loading a sample loader plugin by its name.
	 * @param pstrClassName class name of the plugin sample loader module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @throws RemoteException
	 */
	void setSampleLoaderPluginClass(String pstrClassName)
	throws RemoteException, MARFException;

	/**
	 * Allows setting a loaded sample loader plugin class.
	 * @param poClass class represting a sample loader plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @throws RemoteException
	 */
	void setSampleLoaderPluginClass(Class poClass)
	throws RemoteException, MARFException;

	/**
	 * Allows querying for the current preprocessing plugin class.
	 * @return the internal plugin class
	 * @throws RemoteException
	 */
	Class getSampleLoaderPluginClass()
	throws RemoteException;

	/**
	 * Allows loading a preprocessing plugin by its name.
	 * @param pstrClassName class name of the plugin preprocessing module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @throws RemoteException
	 */
	void setPreprocessingPluginClass(String pstrClassName)
	throws RemoteException, MARFException;

	/**
	 * Allows setting a loaded preprocessing plugin class.
	 * @param poClass class represting a preprocessing plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @throws RemoteException
	 */
	void setPreprocessingPluginClass(Class poClass)
	throws RemoteException, MARFException;

	/**
	 * Allows querying for the current preprocessing plugin class.
	 * @return the internal plugin class
	 * @throws RemoteException
	 */
	Class getPreprocessingPluginClass()
	throws RemoteException;

	/**
	 * Allows loading a feature extraction plugin by its name.
	 * @param pstrClassName class name of the plugin feature extraction module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @throws RemoteException
	 */
	void setFeatureExtractionPluginClass(String pstrClassName)
	throws RemoteException, MARFException;

	/**
	 * Allows setting a loaded feature extraction class plugin class.
	 * @param poClass class represting a feature extraction plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @throws RemoteException
	 */
	void setFeatureExtractionPluginClass(Class poClass)
	throws RemoteException, MARFException;

	/**
	 * Allows querying for the current feature extraction plugin class.
	 * @return the internal plugin class
	 * @throws RemoteException
	 */
	Class getFeatureExtractionPluginClass()
	throws RemoteException;

	/**
	 * Allows loading a classification plugin by its name.
	 * @param pstrClassName class name of the plugin classification module
	 * @throws MARFException if class cannot be loaded for any reason
	 * @throws RemoteException
	 */
	void setClassificationPluginClass(String pstrClassName)
	throws RemoteException, MARFException;

	/**
	 * Allows setting a loaded classification plugin class.
	 * @param poClass class represting a classification plugin object
	 * @throws MARFException if the parameter is <code>null</code>
	 * @throws RemoteException
	 */
	void setClassificationPluginClass(Class poClass)
	throws RemoteException, MARFException;

	/**
	 * Allows querying for the current classification plugin class.
	 * @return the internal plugin class
	 * @throws RemoteException
	 */
	Class getClassificationPluginClass()
	throws RemoteException;

	/**
	 * Returns a string representation of the MARF version.
	 * As of 0.3.0.3 MINOR_REVISION is also returned.
	 * 
	 * @return version String
	 * @throws RemoteException
	 */
	String getVersion()
	throws RemoteException;

	/**
	 * Returns an integer representation of the MARF version.
	 * As of 0.3.0.3, MINOR_REVISION is included into calculations
	 * and the formula changed to begin with 1000 as a MAJOR_VERSION
	 * coefficient.
	 *
	 * @return integer version as <code>MAJOR_VERSION * 1000 + MINOR_VERSION * 100 + REVISION * 10 + MINOR_REVISION</code>
	 *
	 * @see marf.MARF#getIntVersion()
	 * @throws RemoteException
	 */
	int getIntVersion()
	throws RemoteException;

	/**
	 * Retrieves double version of MARF. Unlike the integer version, the double
	 * one begins with 100 and the minor revision is returned after the point,
	 * e.g. 123.4 for 1.2.3.4.
	 *
	 * @return double version as <code>MAJOR_VERSION * 100 + MINOR_VERSION * 10 + REVISION + MINOR_REVISION / 10</code>
	 * @throws RemoteException
	 */
	double getDoubleVersion()
	throws RemoteException;

	/**
	 * Returns a string representation of the current MARF configuration.
	 * @return configuration string
	 * @throws RemoteException
	 */
	String getConfig()
	throws RemoteException;

	/**
	 * Retrieves current <code>Sample</code> reference.
	 * @return Sample object
	 * @throws RemoteException
	 */
	Sample getSample()
	throws RemoteException;

	/**
	 * Retrieves current <code>SampleLoader</code> reference.
	 * @return SampleLoader object
	 * @throws RemoteException
	 */
	ISampleLoader getSampleLoader()
	throws RemoteException;

	/**
	 * Retrieves current <code>SampleLoader</code> reference.
	 * @return SampleLoader object
	 * @throws RemoteException
	 */
	ISampleLoaderRMI getRemoteSampleLoader()
	throws RemoteException;

	/**
	 * Retrieves current <code>Preprocessing</code> reference.
	 * @return Preprocessing object
	 * @throws RemoteException
	 */
	IPreprocessing getPreprocessing()
	throws RemoteException;

	/**
	 * Retrieves current <code>Preprocessing</code> reference.
	 * @return Preprocessing object
	 * @throws RemoteException
	 */
	IPreprocessingRMI getRemotePreprocessing()
	throws RemoteException;

	/**
	 * Retrieves current <code>FeatureExtraction</code> reference.
	 * @return FeatureExtraction object
	 * @throws RemoteException
	 */
	IFeatureExtraction getFeatureExtraction()
	throws RemoteException;

	/**
	 * Retrieves current <code>FeatureExtraction</code> reference.
	 * @return FeatureExtraction object
	 * @throws RemoteException
	 */
	IFeatureExtractionRMI getRemoteFeatureExtraction()
	throws RemoteException;

	/**
	 * Retrieves current <code>Classification</code> reference.
	 * @return Classification object
	 * @throws RemoteException
	 */
	IClassification getClassification()
	throws RemoteException;

	/**
	 * Retrieves current <code>Classification</code> reference.
	 * @return Classification object
	 * @throws RemoteException
	 */
	IClassificationRMI getClassificationRemote()
	throws RemoteException;

	/**
	 * Allows setting all MARF configuration parameters.
	 * @param poConfiguration
	 * @return old configuration
	 * @throws RemoteException
	 * @throws MARFException
	 */
	Configuration setConfiguration(Configuration poConfiguration)
	throws RemoteException, MARFException;
	
	/**
	 * Allows getting all MARF configuration parameters.
	 * @return current configuration
	 * @throws RemoteException
	 */
	Configuration getConfiguration()
	throws RemoteException;

	/**
	 * Queries for the final classification result.
	 * @return integer ID of the indentified subject
	 * @throws RemoteException
	 */
	int queryResultID()
	throws RemoteException;

	/**
	 * Gets the entire Result object of the likely outcome.
	 * @return Result ID and all the stats of the classification
	 * @throws RemoteException
	 */
	Result getResult()
	throws RemoteException;

	/**
	 * Gets the entire collection of results.
	 * @return ResultSet object with one or more results.
	 * @throws RemoteException
	 */
	ResultSet getResultSet()
	throws RemoteException;


	/* API */

	/**
	 * Recognition/Identification mode.
	 * @throws MARFException if there was an error in the pipeline
	 * or classification
	 * @throws RemoteException
	 */
	void recognize()
	throws RemoteException, MARFException;

	/**
	 * Training mode.
	 * @throws MARFException if the subject is unset or there was
	 * an error in training in the underlying classification module
	 * @throws RemoteException
	 */
	void train()
	throws RemoteException, MARFException;

	/**
	 * The core processing pipeline. The pipeline work
	 * through almost entire process up until creation
	 * of the classification module. Then the application's
	 * choice depicts whether to call train() or classify()
	 * on this module via MARF's train() or recognize() methods.
	 * The pipeline is granularly synchronized on each object
	 * that gets instantiated, so no indirect blocking occurs
	 * on MARF itself (e.g. betwen the main thread and the inner
	 * threads of the framework that maybe accessing the MARF
	 * class concurrently).
	 *
	 * @throws MARFException in case any underlying error happens
	 * @see #train()
	 * @see #recognize()
	 * @throws RemoteException
	 */
	void startRecognitionPipeline()
	throws RemoteException, MARFException;

	/**
	 * Checks for all necessary settings to be present.
	 * Specifically, checks whether preprocessing, feature
	 * extraction, classification methods are set as well as
	 * the audio sample format. The filename of the sample
	 * or a directory must also be present.
	 *
	 * @throws MARFException if any of the settings are unset
	 * @throws RemoteException
	 */
	void checkSettings()
	throws RemoteException, MARFException;
	
	/**
	 * Meant to provide implementation of the buffered sample processing for large samples.
	 * Not implemented.
	 * @throws NotImplementedException
	 * @throws RemoteException
	 */
	void streamedRecognition()
	throws RemoteException;


	/**
	 * <p>Class NLP is more related to the Natural Language Processing
	 * part of MARF.</p>
	 *
	 * @author Serguei Mokhov
	 */
	public interface NLP
	extends IRMIServer, Cloneable
	{
		/**
		 * Retrieves current smoothing method.
		 * @return inner smoothing method
		 * @throws RemoteException
		 */
		int getSmoothingMethod()
		throws RemoteException;

		/**
		 * Sets current smoothing method.
		 * @param piSmoothingMethod new smoothing method to use
		 * @throws NLPException if the parameter is outside the valid range
		 * @see EStatisticalEstimators#MIN_STATS_ESTIMATOR
		 * @see EStatisticalEstimators#MAX_STATS_ESTIMATOR
		 * @throws RemoteException
		 */
		void setSmoothingMethod(final int piSmoothingMethod)
		throws NLPException, RemoteException;

		/**
		 * Retrieves current n-gram modeld.
		 * @return inner n-gram model
		 * @throws RemoteException
		 */
		int getNgramModel()
		throws RemoteException;

		/**
		 * Sets current n-gram model.
		 * @param piNgramModel new n-gram model to use
		 * @throws NLPException if the parameter is outside the valid range
		 * @see ENgramModels#MIN_NGRAM_MODEL
		 * @see ENgramModels#MAX_NGRAM_MODEL
		 * @throws RemoteException
		 */
		void setNgramModel(final int piNgramModel)
		throws NLPException, RemoteException;

		/**
		 * Retrieves current language being processed.
		 * @return inner smoothing method
		 * @throws RemoteException
		 */
		String getLanguage()
		throws RemoteException;

		/**
		 * Sets current processed language.
		 * @param pstrLanguages new language value
		 * @throws NLPException if the parameter is null or empty
		 * @throws RemoteException
		 */
		void setLanguage(String pstrLanguages)
		throws NLPException, RemoteException;
	} // interface NLP
}

// EOF
