package marf.net.server.rmi.Preprocessing;

import java.rmi.RemoteException;

import marf.Preprocessing.PreprocessingException;
import marf.Storage.Sample;
import marf.net.server.rmi.IRMIServer;
import marf.util.MARFException;


/**
 * <p>Remote Preprocessing Interface.</p>
 *
 * $Id: IPreprocessingRMI.java,v 1.4 2006/08/16 03:34:19 mokhov Exp $
 *
 * @author Serguei Mokhov
 * @version $Revision: 1.4 $
 * @since 0.3.0.6
 */
public interface IPreprocessingRMI
extends IRMIServer, Cloneable
{
	/**
	 * Interface source code revision.
	 */
	String MARF_INTERFACE_CODE_REVISION = "$Revision: 1.4 $";

	/**
	 * @param piPreprocessingMethod
	 * @param poSample
	 * @return
	 * @throws RemoteException
	 * @throws PreprocessingException
	 */
	boolean init(int piPreprocessingMethod, Sample poSample)
	throws RemoteException, MARFException;

	/**
	 * Derivatives must implement this method to do general
	 * preprocessing and perhaps calling <code>removeNoise()</code>
	 * and <code>removeSilence()</code>.
	 *
	 * @return boolean that sample has changed as a result of preprocessing
	 *
	 * @throws PreprocessingException if an error occurred while preprocessing
	 *
	 * @see #removeNoise()
	 * @see #removeSilence()
	 */
	boolean preprocess()
	throws RemoteException, PreprocessingException;

	/**
	 * Derivatives should implement this method to remove noise from the sample.
	 *
	 * @return boolean that sample has changed (noise removed)
	 * @throws PreprocessingException declared but never thrown
	 */
	boolean removeNoise()
	throws RemoteException, PreprocessingException;

	/**
	 * Derivatives should implement this method to remove silence.
	 *
	 * @return boolean that sample has changed (silence removed)
	 * @throws PreprocessingException declared but never thrown
	 */
	boolean removeSilence()
	throws RemoteException, PreprocessingException;

	/**
	 * Normalization of entire incoming samples by amplitude.
	 *
	 * @return <code>true</code> if the sample has been successfully normalized;
	 * <code>false</code> otherwise
	 * @throws PreprocessingException if internal sample reference is null
	 */
	boolean normalize()
	throws RemoteException, PreprocessingException;

	/**
	 * Normalization of incoming samples by amplitude starting from certain index.
	 * Useful in case where only the last portion of a sample needs to be normalized.
	 *
	 * @param piIndexFrom sample array index to start normalization from
	 *
	 * @return <code>true</code> if the sample has been successfully normalized;
	 * <code>false</code> otherwise
	 * @throws PreprocessingException if internal sample reference is null
	 *
	 * @see #normalize(int, int)
	 */
	boolean normalize(int piIndexFrom)
	throws RemoteException, PreprocessingException;

	/**
	 * Normalization of incoming samples by amplitude between specified indexes.
	 * Useful in case where only a portion of a sample needs to be normalized.
	 *
	 * @param piIndexFrom sample array index to start normalization from
	 * @param piIndexTo sample array index to end normalization at
	 *
	 * @return <code>true</code> if the sample has been successfully normalized;
	 * <code>false</code> otherwise
	 * @throws PreprocessingException if internal sample reference is null or one or
	 * both indexes are out of range
	 */
	boolean normalize(int piIndexFrom, int piIndexTo)
	throws RemoteException, PreprocessingException;

	/**
	 * Derivatives implement this method to crop arbitrary
	 * part of the audio sample.
	 *
	 * @param pdStartingFrequency double Frequency to start to crop from
	 * @param pdEndFrequency double Frequency to crop the sample to
	 *
	 * @return boolean <code>true</code> - cropped, <code>false</code> - not
	 * @throws PreprocessingException declared, but is never thrown
	 */
	boolean cropAudio(double pdStartingFrequency, double pdEndFrequency)
	throws RemoteException, PreprocessingException;

	/**
	 * Returns enclosed sample.
	 * @return Sample object
	 */
	Sample getSample()
	throws RemoteException;

	/**
	 * Allows setting a sample object reference.
	 * @param poSample new sample object
	 */
	void setSample(Sample poSample)
	throws RemoteException;
	
	/**
	 * Enable cloning of preprocessing modules in accordance
	 * with the Cloneable interface.
	 * @return a copy of the object implementing this interface
	 * @throws CloneNotSupportedException if for some reason cloning is unsupported. 
	 */
//    Object clone()
 //   throws /*RemoteException,*/ CloneNotSupportedException;
}

// EOF
